{% load staticfiles %}
{% if DEBUG %}
    <script src="{% static 'js/jquery.js' %}"></script>
    <script src="{% static 'js/csrf_token.js' %}"></script>
    <script src="{% static 'js/json2.js' %}"></script>
    <script src="{% static 'js/underscore.js' %}"></script>
    <script src="{% static 'js/backbone.js' %}"></script>
    <script src="{% static 'js/backbone.marionette.js' %}"></script>
    <script src="{% static 'js/backbone.radio.js' %}"></script>
{% else %}
    <script src="{% static 'js/jquery.min.js' %}"></script>
    <script src="{% static 'js/csrf_token.js' %}"></script>
    <script src="{% static 'js/json2.js' %}"></script>
    <script src="{% static 'js/underscore.min.js' %}"></script>
    <script src="{% static 'js/backbone.min.js' %}"></script>
    <script src="{% static 'js/backbone.marionette.min.js' %}"></script>
    <script src="{% static 'js/backbone.radio.min.js' %}"></script>
{% endif %}
{% block extra %}
{% endblock %}
